#include <errno.h>
#include <stdio.h>
#include <string.h>

#include <libgsloop.h>
#include <gsloop_types.h>
#include <gsloop_con.h>


int _gsl_debuglevel = 100;


int gsloop_init(gsloop_t	*gsloop_anon,
		gsloop_attr_t	*attr)
{
	struct gsloop	*gsloop = (struct gsloop *)gsloop_anon;
	int		err;

	if (!gsloop_anon || !attr)
		return -EINVAL;

	memset(gsloop, 0, sizeof(struct gsloop));

	memcpy(&gsloop->attr, attr, sizeof(struct gsloop_attr));

	err = rn_init(&gsloop->rn, gsloop->attr.max_fd);
	if (err) {
		err = -err;	/* rn returns "errno" */
		goto exit_0;
	}

	err = gsloop_con_init(gsloop);
	if (err)
		goto exit_1;

	/* err = gsloop_sig_init(); */

	return 0;

 exit_1:
	rn_shutdown(&gsloop->rn);
 exit_0:
	return err;
}


int gsloop_destroy(gsloop_t	*gsloop_anon)
{
	struct gsloop	*gsloop = (struct gsloop *)gsloop_anon;
	int		i, err = 0, ret;

	if (!gsloop_anon)
		return -EINVAL;

	for (i = 0; i < gsloop->con_alloced; i++) {
		if (gsloop->con_list[i].common.fd != 0) {
			ret = gsloop->con_list[i].common.close(gsloop, &gsloop->con_list[i]);
			if (ret) {
				err = ret;
				dprintf(LEVEL_ERROR, "failed to close connection nr %d, err: %d\n",
					i, err);
			}
		}
	}

	rn_shutdown(&gsloop->rn);

	return err;
}


int gsloop_dispatch(gsloop_t		*gsloop_anon,
		    unsigned int	timeout)
{
	struct gsloop	*gsloop = (struct gsloop *)gsloop_anon;
	int		err;

	if (!gsloop_anon)
		return -EINVAL;

	err = rn_waitAndDispatchEvents(&gsloop->rn, timeout);

	return err;
}
